COMMAND_CATEGORIES = [
    "buildAnaDroid.commands.execute_code",
    "buildAnaDroid.commands.file_operations",
    "buildAnaDroid.commands.system",
    "buildAnaDroid.commands.gradle_build_error_solver",
]